"""
This file defines variables for the modelled legislation.

A variable is a property of an Entity such as a 人物, a 世帯…

See https://openfisca.org/doc/key-concepts/variables.html
"""

from functools import cache

import numpy as np
# Import from openfisca-core the Python objects used to code the legislation in OpenFisca
from openfisca_core.holders import set_input_divide_by_period
from openfisca_core.periods import DAY, period
from openfisca_core.variables import Variable
from openfisca_japan import COUNTRY_DIR
# Import the Entities specifically defined for this tax and benefit system
from openfisca_japan.entities import 世帯, 人物
from openfisca_japan.variables.障害.愛の手帳 import 愛の手帳等級パターン
from openfisca_japan.variables.障害.療育手帳 import 療育手帳等級パターン
from openfisca_japan.variables.障害.精神障害者保健福祉手帳 import 精神障害者保健福祉手帳等級パターン
from openfisca_japan.variables.障害.身体障害者手帳 import 身体障害者手帳等級パターン


# NOTE: 項目数が多い金額表は可読性の高いCSV形式としている。


@cache
def 配偶者控除額表():
    """
    csvファイルから値を読み込み

    配偶者控除額表()[配偶者の所得区分, 納税者本人の所得区分] の形で参照可能
    """
    return np.genfromtxt(COUNTRY_DIR + "/assets/所得/配偶者控除額.csv",
                  delimiter=",", skip_header=1, dtype="int64")[np.newaxis, 1:]


@cache
def 配偶者特別控除額表():
    """
    csvファイルから値を読み込み

    配偶者特別控除額表()[配偶者の所得区分, 納税者本人の所得区分] の形で参照可能
    """
    return np.genfromtxt(COUNTRY_DIR + "/assets/所得/配偶者特別控除額.csv",
                  delimiter=",", skip_header=1, dtype="int64")[:, 1:]


@cache
def 老人控除対象配偶者_配偶者控除額表():
    """
    csvファイルから値を読み込み

    老人控除対象配偶者_配偶者控除額表()[配偶者の所得区分, 納税者本人の所得区分] の形で参照可能
    """
    return np.genfromtxt(COUNTRY_DIR + "/assets/所得/配偶者控除額_老人控除対象配偶者.csv",
                         delimiter=",", skip_header=1, dtype="int64")[np.newaxis, 1:]


class 給与所得控除額(Variable):
    value_type = float
    entity = 人物
    # NOTE: Variable自体は1年ごとに定義されるが、特定の日付における各種手当に計算できるように DAY で定義
    definition_period = DAY
    # Optional attribute. Allows user to declare this variable for a year.
    # OpenFisca will spread the yearly 金額 over the days contained in the year.
    set_input = set_input_divide_by_period
    label = "人物の収入に対する給与所得控除額"

    def formula_2020_01_01(対象人物, 対象期間, _parameters):
        収入 = 対象人物("収入", 対象期間)

        return np.select([収入 <= 1625000, 収入 <= 1800000, 収入 <= 3600000, 収入 <= 6600000, 収入 <= 8500000],
                         [float(550000), 収入 * 0.4 - 100000, 収入 * 0.3 + 80000, 収入 * 0.2 + 440000, 収入 * 0.1 + 1100000],
                         float(1950000))

    # TODO: 必要であれば平成28(2016)年より前の計算式も追加
    def formula_2017_01_01(対象人物, 対象期間, _parameters):
        収入 = 対象人物("収入", 対象期間)

        return np.select([収入 <= 1625000, 収入 <= 1800000, 収入 <= 3600000, 収入 <= 6600000, 収入 <= 10000000],
                         [float(650000), 収入 * 0.4, 収入 * 0.3 + 180000, 収入 * 0.2 + 540000, 収入 * 0.1 + 1200000],
                         float(2200000))


class 障害者控除(Variable):
    value_type = float
    entity = 人物
    definition_period = DAY
    label = "障害者控除額"
    reference = "https://www.nta.go.jp/taxes/shiraberu/taxanswer/shotoku/1160.htm"

    def formula(対象人物, 対象期間, _parameters):
        # 自身が扶養に入っている、または同一生計配偶者である場合、納税者（世帯主）が控除を受ける
        扶養親族である = 対象人物("扶養親族である", 対象期間)
        同一生計配偶者である = 対象人物("同一生計配偶者である", 対象期間)
        被扶養者である = 扶養親族である + 同一生計配偶者である

        所得 = 対象人物("所得", 対象期間)
        所得降順 = 対象人物.get_rank(対象人物.世帯, -所得)
        # NOTE: 便宜上、被扶養者は所得が最も高い世帯員の扶養に入るとする
        所得が最も高い世帯員である = 所得降順 == 0

        控除対象額 = 対象人物("障害者控除対象額", 対象期間)
        # NOTE: 異なる人物に対する値であるため、人物ではなく世帯ごとに集計（でないと「扶養者である」と要素がずれてしまい計算できない)
        被扶養者の合計控除額 = 対象人物.世帯.sum(被扶養者である * 控除対象額)

        # 最も所得が高い世帯員ではないが、一定以上の所得がある場合
        扶養に入っていない納税者である = np.logical_not(所得が最も高い世帯員である) * np.logical_not(被扶養者である)
        # 被扶養者は控除を受けない（扶養者が代わりに控除を受けるため）
        return 所得が最も高い世帯員である * (被扶養者の合計控除額 + 控除対象額) + 扶養に入っていない納税者である * 控除対象額


class 障害者控除対象額(Variable):
    value_type = float
    entity = 人物
    definition_period = DAY
    label = "障害者控除額の対象額"
    reference = "https://www.nta.go.jp/taxes/shiraberu/taxanswer/shotoku/1160.htm"
    documentation = """
    実際の控除額は同一生計配偶者、扶養親族が該当する場合にも加算される
    """

    def formula(対象人物, 対象期間, parameters):
        # 障害者控除額は対象人物ごとに算出される
        # https://www.city.hirakata.osaka.jp/kosodate/0000000544.html
        同居特別障害者控除対象 = 対象人物("同居特別障害者控除対象", 対象期間)
        # 重複して該当しないよう、同居特別障害者控除対象の場合を除外
        特別障害者控除対象 = 対象人物("特別障害者控除対象", 対象期間) * np.logical_not(同居特別障害者控除対象)
        障害者控除対象 = 対象人物("障害者控除対象", 対象期間)

        同居特別障害者控除額 = parameters(対象期間).所得.同居特別障害者控除額
        特別障害者控除額 = parameters(対象期間).所得.特別障害者控除額
        障害者控除額 = parameters(対象期間).所得.障害者控除額

        return 同居特別障害者控除対象 * 同居特別障害者控除額 + 特別障害者控除対象 * 特別障害者控除額 + 障害者控除対象 * 障害者控除額


class 障害者控除対象(Variable):
    value_type = bool
    entity = 人物
    definition_period = DAY
    label = "障害者控除の対象になるか否か"
    reference = "https://www.nta.go.jp/taxes/shiraberu/taxanswer/shotoku/1160.htm"

    def formula(対象人物, 対象期間, _parameters):
        身体障害者手帳等級 = 対象人物("身体障害者手帳等級", 対象期間)
        精神障害者保健福祉手帳等級 = 対象人物("精神障害者保健福祉手帳等級", 対象期間)
        療育手帳等級 = 対象人物("療育手帳等級", 対象期間)
        愛の手帳等級 = 対象人物("愛の手帳等級", 対象期間)

        特別障害者控除対象 = 対象人物("特別障害者控除対象", 対象期間)

        障害者控除対象 = \
            ~特別障害者控除対象 *  \
            ((身体障害者手帳等級 != 身体障害者手帳等級パターン.無)
             + (精神障害者保健福祉手帳等級 != 精神障害者保健福祉手帳等級パターン.無)
         + (療育手帳等級 != 療育手帳等級パターン.無)
                + (愛の手帳等級 != 愛の手帳等級パターン.無))

        return 障害者控除対象


class 特別障害者控除対象(Variable):
    value_type = bool
    entity = 人物
    definition_period = DAY
    label = "特別障害者控除の対象になるか否か"
    reference = "https://www.nta.go.jp/taxes/shiraberu/taxanswer/shotoku/1160.htm"

    def formula(対象人物, 対象期間, _parameters):
        身体障害者手帳等級 = 対象人物("身体障害者手帳等級", 対象期間)
        精神障害者保健福祉手帳等級 = 対象人物("精神障害者保健福祉手帳等級", 対象期間)
        療育手帳等級 = 対象人物("療育手帳等級", 対象期間)
        愛の手帳等級 = 対象人物("愛の手帳等級", 対象期間)

        特別障害者控除対象 = \
            (身体障害者手帳等級 == 身体障害者手帳等級パターン.一級) + \
            (身体障害者手帳等級 == 身体障害者手帳等級パターン.二級) + \
            (精神障害者保健福祉手帳等級 == 精神障害者保健福祉手帳等級パターン.一級) + \
            (療育手帳等級 == 療育手帳等級パターン.A) + \
            (愛の手帳等級 == 愛の手帳等級パターン.一度) + \
            (愛の手帳等級 == 愛の手帳等級パターン.二度)

        return 特別障害者控除対象


class 同居特別障害者控除対象(Variable):
    value_type = bool
    entity = 人物
    definition_period = DAY
    label = "同居特別障害者控除の対象になるか否か"
    reference = "https://www.nta.go.jp/taxes/shiraberu/taxanswer/shotoku/1160.htm"

    def formula(対象人物, 対象期間, _parameters):
        特別障害者控除対象 = 対象人物("特別障害者控除対象", 対象期間)
        同一生計配偶者である = 対象人物("同一生計配偶者である", 対象期間)
        扶養親族である = 対象人物("扶養親族である", 対象期間)

        # TODO: 「同居していない親族」も世帯内で扱うようになったら以下の判定追加（現状フロントエンドでは同居している親族しか扱っていない）
        # 「納税者自身、配偶者、その納税者と生計を一にする親族のいずれかとの同居を常況としている」
        return 特別障害者控除対象 * (同一生計配偶者である | 扶養親族である)


class ひとり親控除(Variable):
    value_type = float
    entity = 人物
    definition_period = DAY
    label = "ひとり親控除額"
    reference = "https://www.nta.go.jp/taxes/shiraberu/taxanswer/shotoku/1171.htm"

    def formula_2020_01_01(対象人物, 対象期間, parameters):
        所得 = 対象人物("所得", 対象期間)
        # 児童扶養手当の対象と異なり、父母の遺棄・DV等は考慮しない
        # (参考：児童扶養手当 https://www.city.hirakata.osaka.jp/0000026828.html)
        親である = 対象人物.has_role(世帯.親)
        子である = 対象人物.has_role(世帯.子)
        対象ひとり親 = (対象人物.世帯.sum(親である) == 1) * (対象人物.世帯.sum(子である) >= 1)
        ひとり親控除額 = parameters(対象期間).所得.ひとり親控除額
        ひとり親控除_所得制限額 = parameters(対象期間).所得.ひとり親控除_所得制限額

        return 親である * ひとり親控除額 * 対象ひとり親 * (所得 < ひとり親控除_所得制限額)


class 寡婦控除(Variable):
    value_type = float
    entity = 人物
    definition_period = DAY
    label = "寡婦控除額"
    reference = "https://www.nta.go.jp/taxes/shiraberu/taxanswer/shotoku/1170.htm"

    def formula_2020_01_01(対象人物, 対象期間, parameters):
        所得 = 対象人物("所得", 対象期間)
        寡婦 = 対象人物("寡婦", 対象期間)
        寡婦控除額 = parameters(対象期間).所得.寡婦控除額
        寡婦控除_所得制限額 = parameters(対象期間).所得.寡婦控除_所得制限額

        return 寡婦控除額 * 寡婦 * (所得 <= 寡婦控除_所得制限額)


class 学生(Variable):
    value_type = bool
    default_value = False
    entity = 人物
    definition_period = DAY
    label = "小・中・高校、大学、専門学校、職業訓練学校等の学生"
    reference = "https://www.nta.go.jp/taxes/shiraberu/taxanswer/shotoku/1175.htm"


class 勤労学生控除(Variable):
    value_type = float
    entity = 人物
    definition_period = DAY
    label = "勤労学生控除"
    reference = "https://www.nta.go.jp/taxes/shiraberu/taxanswer/shotoku/1175.htm"

    def formula(対象人物, 対象期間, parameters):
        所得 = 対象人物("所得", 対象期間)
        学生である = 対象人物("学生", 対象期間)
        勤労学生控除額 = parameters(対象期間).所得.勤労学生控除額
        勤労学生_所得制限額 = parameters(対象期間).所得.勤労学生_所得制限額
        所得条件 = (所得 > 0) * (所得 <= 勤労学生_所得制限額)

        return 所得条件 * 学生である * 勤労学生控除額


class 同一生計配偶者である(Variable):
    value_type = bool
    default_value = False
    entity = 人物
    definition_period = DAY
    label = "同一生計配偶者であるか否か"
    reference = "https://www.nta.go.jp/taxes/shiraberu/shinkoku/tebiki/2022/03/order3/yogo/3-3_y15.htm"

    def formula(対象人物, 対象期間, parameters):
        所得 = 対象人物("所得", 対象期間)
        # 所得が高いほうが控除を受ける対象となる
        所得順位 = 対象人物.get_rank(対象人物.世帯, -所得, condition=対象人物.has_role(世帯.親))
        配偶者である = (所得順位 == 1)  # 親のうち所得順位が低い方が配偶者

        同一生計配偶者_所得制限額 = parameters(対象期間).所得.同一生計配偶者_所得制限額
        return 配偶者である * (所得 <= 同一生計配偶者_所得制限額)


class 配偶者控除(Variable):
    value_type = float
    entity = 人物
    definition_period = DAY
    label = "配偶者控除"
    reference = "https://www.nta.go.jp/taxes/shiraberu/taxanswer/shotoku/1191.htm"
    documentation = """
    配偶者特別控除とは異なる。
    配偶者の所得が配偶者控除の所得制限を超えた場合でも、配偶者特別控除が適用される可能性がある。
    """

    def formula(対象人物, 対象期間, parameters):
        # 所得が高いほうが控除を受ける対象となる
        所得一覧 = 対象人物("所得", 対象期間)
        所得降順 = 対象人物.get_rank(対象人物.世帯, - 所得一覧, condition=対象人物.has_role(世帯.親))
        控除対象者である = (所得降順 == 0) * 対象人物.has_role(世帯.親)
        控除対象者の配偶者である = (所得降順 == 1) * 対象人物.has_role(世帯.親)
        控除対象者の所得 = 所得一覧 * 控除対象者である
        # NOTE: 異なる人物に対する値であるため、人物ではなく世帯ごとに集計（でないと「控除対象者の所得」と要素がずれてしまい計算できない)
        控除対象者の配偶者の所得 = 対象人物.世帯.sum(所得一覧 * 控除対象者の配偶者である)

        # 複数世帯の前世帯員のうち、自分または配偶者のroleをもつ世帯員がTrueのarray
        所得一覧 = 対象人物("所得", 対象期間)  # (全世帯員数)の長さのarray

        同一生計配偶者_所得制限額 = parameters(対象期間).所得.同一生計配偶者_所得制限額
        控除対象者の配偶者の所得区分 = np.select(
            [控除対象者の配偶者の所得 <= 同一生計配偶者_所得制限額],
            [0],
            -1).astype(int)  # intにできるようデフォルトをNoneではなく-1

        控除対象者の所得区分 = np.select(
            [控除対象者の所得 <= 9000000,
             (控除対象者の所得 > 9000000) * (控除対象者の所得 <= 9500000),
             (控除対象者の所得 > 9500000) * (控除対象者の所得 <= 10000000)],  # 複数世帯のarrayのためand, orの代わりに *. +
            list(range(3)),
            -1).astype(int)  # intにできるようデフォルトをNoneではなく-1

        対象所得区分に該当する = (控除対象者の所得区分 != -1) * (控除対象者の配偶者の所得区分 != -1)  # 控除条件

        # NOTE: その年の12/31時点の年齢を参照
        # https://www.nta.go.jp/taxes/shiraberu/taxanswer/yogo/senmon.htm#word5
        該当年12月31日 = period(f"{対象期間.start.year}-12-31")
        該当年12月31日の年齢一覧 = 対象人物("年齢", 該当年12月31日)
        控除対象者の配偶者の年齢 = 該当年12月31日の年齢一覧 * 控除対象者の配偶者である
        # NOTE: 自分ではない人物についての計算のため、世帯で計算（でないと要素がずれてしまい計算できない）
        配偶者が老人控除対象である = 対象人物.世帯.sum(控除対象者の配偶者の年齢 >= 70)
        老人控除対象配偶者控除額 = 老人控除対象配偶者_配偶者控除額表()[控除対象者の配偶者の所得区分, 控除対象者の所得区分]

        通常配偶者控除額 = 配偶者控除額表()[控除対象者の配偶者の所得区分, 控除対象者の所得区分]

        配偶者控除額 = np.logical_not(配偶者が老人控除対象である) * 通常配偶者控除額 + 配偶者が老人控除対象である * 老人控除対象配偶者控除額

        配偶者がいる = 対象人物.世帯.sum(対象人物.has_role(世帯.親)) == 2  # 控除条件

        return 控除対象者である * 配偶者がいる * 対象所得区分に該当する * 配偶者控除額


class 配偶者特別控除(Variable):
    value_type = float
    entity = 人物
    definition_period = DAY
    label = "配偶者特別控除"
    reference = "https://www.nta.go.jp/taxes/shiraberu/taxanswer/shotoku/1195.htm"

    def formula(対象人物, 対象期間, _parameters):
        # 所得が高いほうが控除を受ける対象となる
        所得一覧 = 対象人物("所得", 対象期間)
        所得降順 = 対象人物.get_rank(対象人物.世帯, - 所得一覧, condition=対象人物.has_role(世帯.親))
        控除対象者である = (所得降順 == 0) * 対象人物.has_role(世帯.親)
        控除対象者の配偶者である = (所得降順 == 1) * 対象人物.has_role(世帯.親)
        控除対象者の所得 = 所得一覧 * 控除対象者である
        # NOTE: 異なる人物に対する値であるため、人物ではなく世帯ごとに集計（でないと「控除対象者の所得」と要素がずれてしまい計算できない)
        控除対象者の配偶者の所得 = 対象人物.世帯.sum(所得一覧 * 控除対象者の配偶者である)

        控除対象者の所得区分 = np.select(
            [控除対象者の所得 <= 9000000,
             (控除対象者の所得 > 9000000) * (控除対象者の所得 <= 9500000),
             (控除対象者の所得 > 9500000) * (控除対象者の所得 <= 10000000)],
            list(range(3)),
            -1).astype(int)

        控除対象者の配偶者の所得区分 = np.select(
            [(控除対象者の配偶者の所得 > 480000) * (控除対象者の配偶者の所得 <= 950000),
             (控除対象者の配偶者の所得 > 950000) * (控除対象者の配偶者の所得 <= 1000000),
             (控除対象者の配偶者の所得 > 1000000) * (控除対象者の配偶者の所得 <= 1050000),
             (控除対象者の配偶者の所得 > 1050000) * (控除対象者の配偶者の所得 <= 1100000),
             (控除対象者の配偶者の所得 > 1100000) * (控除対象者の配偶者の所得 <= 1150000),
             (控除対象者の配偶者の所得 > 1150000) * (控除対象者の配偶者の所得 <= 1200000),
             (控除対象者の配偶者の所得 > 1200000) * (控除対象者の配偶者の所得 <= 1250000),
             (控除対象者の配偶者の所得 > 1250000) * (控除対象者の配偶者の所得 <= 1300000),
             (控除対象者の配偶者の所得 > 1300000) * (控除対象者の配偶者の所得 <= 1330000)],
            list(range(9)),
            -1).astype(int)

        対象所得区分に該当する = (控除対象者の所得区分 != -1) * (控除対象者の配偶者の所得区分 != -1)  # 控除条件

        配偶者がいる = 対象人物.世帯.sum(対象人物.has_role(世帯.親)) == 2  # 控除条件

        return 控除対象者である * 配偶者がいる * 対象所得区分に該当する * 配偶者特別控除額表()[控除対象者の配偶者の所得区分, 控除対象者の所得区分]


class 扶養控除(Variable):
    value_type = float
    entity = 人物
    definition_period = DAY
    label = "扶養控除"
    reference = "https://www.nta.go.jp/taxes/shiraberu/taxanswer/shotoku/1180.htm"

    def formula(対象人物, 対象期間, parameters):
        扶養親族である = 対象人物("扶養親族である", 対象期間)

        # NOTE: その年の12/31時点の年齢を参照
        # https://www.nta.go.jp/taxes/shiraberu/taxanswer/yogo/senmon.htm#word5
        該当年12月31日 = period(f"{対象期間.start.year}-12-31")
        年齢 = 対象人物("年齢", 該当年12月31日)

        控除対象扶養親族である = 扶養親族である * (年齢 >= 16)

        特定扶養親族である = 控除対象扶養親族である * (年齢 >= 19) * (年齢 < 23)
        老人扶養親族である = 控除対象扶養親族である * (年齢 >= 70)

        # NOTE: 入院中の親族は同居扱いだが老人ホーム等への入居は除く
        # TODO: 「同居していない親族」も世帯内で扱うようになったら同居老親かどうかの判定追加
        介護施設入所中 = 対象人物("介護施設入所中", 対象期間)
        同居している老人扶養親族である = 老人扶養親族である * np.logical_not(介護施設入所中)
        同居していない老人扶養親族である = 老人扶養親族である * 介護施設入所中

        # NOTE: np.selectのcondlistは最初に該当した条件で計算される
        扶養控除額 = np.select(
            [特定扶養親族である,
             同居している老人扶養親族である,
             同居していない老人扶養親族である,
             控除対象扶養親族である],
            [parameters(対象期間).所得.扶養控除_特定扶養親族,
             parameters(対象期間).所得.扶養控除_老人扶養親族_同居老親等,
             parameters(対象期間).所得.扶養控除_老人扶養親族_同居老親等以外の者,
             parameters(対象期間).所得.扶養控除_一般],
            0)

        所得 = 対象人物("所得", 対象期間)
        所得降順 = 対象人物.get_rank(対象人物.世帯, -所得)
        # NOTE: 便宜上所得が最も多い世帯員が扶養者であるとする
        扶養者である = 所得降順 == 0

        return 扶養者である * 対象人物.世帯.sum(扶養控除額)


class 扶養親族である(Variable):
    value_type = bool
    default_value = False
    entity = 人物
    definition_period = DAY
    label = "扶養親族であるか否か"
    reference = "https://www.nta.go.jp/taxes/shiraberu/taxanswer/yogo/senmon.htm"

    def formula(対象人物, 対象期間, parameters):
        扶養親族所得金額 = parameters(対象期間).所得.扶養親族所得金額
        所得 = 対象人物("所得", 対象期間)
        親である = 対象人物.has_role(世帯.親)

        # 扶養親族に配偶者は含まれない。(親等の児童以外を扶養する場合はそれらも含む必要あり)
        # 扶養親族の定義(参考): https://www.nta.go.jp/taxes/shiraberu/taxanswer/shotoku/1180.htm
        return np.logical_not(親である) * (所得 <= 扶養親族所得金額)


class 扶養人数(Variable):
    value_type = int
    entity = 世帯
    definition_period = DAY
    label = "扶養人数"

    def formula(対象世帯, 対象期間, parameters):
        扶養親族である = 対象世帯.members("扶養親族である", 対象期間)
        # この時点でndarrayからスカラーに変換しても、他から扶養人数を取得する際はndarrayに変換されて返されてしまう
        return 対象世帯.sum(扶養親族である)


class 控除後世帯高所得(Variable):
    value_type = float
    entity = 世帯
    definition_period = DAY
    label = "各種控除が適用された後の世帯高所得額"
    reference = "https://www.city.himeji.lg.jp/waku2child/0000013409.html"

    def formula(対象世帯, 対象期間, _parameters):
        所得 = 対象世帯.members("所得", 対象期間)
        所得降順 = 対象世帯.get_rank(対象世帯, -所得)
        # 最も所得が大きい世帯員を対象とすることで、世帯高所得を算出している
        対象者である = 所得降順 == 0

        控除後所得 = 対象世帯.members("控除後所得", 対象期間)
        return 対象世帯.sum(対象者である * 控除後所得)


class 控除後所得(Variable):
    value_type = float
    entity = 人物
    definition_period = DAY
    label = "各種控除が適用された後の所得額"
    reference = "https://www.city.himeji.lg.jp/waku2child/0000013409.html"

    def formula(対象人物, 対象期間, parameters):
        所得 = 対象人物("所得", 対象期間)
        社会保険料 = parameters(対象期間).所得.社会保険料相当額
        給与所得及び雑所得からの控除額 = parameters(対象期間).所得.給与所得及び雑所得からの控除額
        障害者控除 = 対象人物("障害者控除", 対象期間)
        ひとり親控除 = 対象人物("ひとり親控除", 対象期間)
        寡婦控除 = 対象人物("寡婦控除", 対象期間)
        勤労学生控除 = 対象人物("勤労学生控除", 対象期間)

        # 他の控除（雑損控除・医療費控除等）は定額でなく実費を元に算出するため除外する
        総控除額 = 社会保険料 + 給与所得及び雑所得からの控除額 + 障害者控除 + ひとり親控除 + 寡婦控除 + 勤労学生控除

        # 負の数にならないよう、0円未満になった場合は0円に補正
        return np.clip(所得 - 総控除額, 0.0, None)


class 児童手当の控除後世帯高所得(Variable):
    value_type = float
    entity = 世帯
    definition_period = DAY
    label = "各種控除が適用された後の児童手当における世帯高所得額"
    reference = "https://www.city.himeji.lg.jp/waku2child/0000013409.html"
    documentation = """
    所得税等の控除額とは異なる。
    https://www.nta.go.jp/publication/pamph/koho/kurashi/html/01_1.htm
    """

    def formula(対象世帯, 対象期間, _parameters):
        所得 = 対象世帯.members("所得", 対象期間)
        所得降順 = 対象世帯.get_rank(対象世帯, -所得)
        # 最も所得が大きい世帯員を対象とすることで、世帯高所得を算出している
        対象者である = 所得降順 == 0

        控除後所得 = 対象世帯.members("児童手当の控除後所得", 対象期間)
        return 対象世帯.sum(対象者である * 控除後所得)


class 児童手当の控除後所得(Variable):
    value_type = float
    entity = 人物
    definition_period = DAY
    label = "各種控除が適用された後の児童手当における所得額"
    reference = "https://www.city.himeji.lg.jp/waku2child/0000013409.html"
    documentation = """
    所得税等の控除額とは異なる。
    https://www.nta.go.jp/publication/pamph/koho/kurashi/html/01_1.htm
    """

    def formula(対象人物, 対象期間, parameters):
        所得 = 対象人物("所得", 対象期間)
        社会保険料 = parameters(対象期間).所得.社会保険料相当額
        給与所得及び雑所得からの控除額 = parameters(対象期間).所得.給与所得及び雑所得からの控除額
        障害者控除 = 対象人物("障害者控除", 対象期間)
        ひとり親控除 = 対象人物("ひとり親控除", 対象期間)
        寡婦控除 = 対象人物("寡婦控除", 対象期間)
        勤労学生控除 = 対象人物("勤労学生控除", 対象期間)

        # 他の控除（雑損控除・医療費控除等）は定額でなく実費を元に算出するため除外する

        総控除額 = 社会保険料 + 給与所得及び雑所得からの控除額 + 障害者控除 + ひとり親控除 + 寡婦控除 + 勤労学生控除

        # 負の数にならないよう、0円未満になった場合は0円に補正
        return np.clip(所得 - 総控除額, 0.0, None)


class 児童扶養手当の控除後世帯高所得(Variable):
    value_type = float
    entity = 世帯
    definition_period = DAY
    label = "各種控除が適用された後の児童扶養手当の世帯高所得額"
    reference = "https://www.city.otsu.lg.jp/soshiki/015/1406/g/jidofuyoteate/1389538447829.html"

    def formula(対象世帯, 対象期間, _parameters):
        所得 = 対象世帯.members("所得", 対象期間)
        所得降順 = 対象世帯.get_rank(対象世帯, -所得)
        # 最も所得が大きい世帯員を対象とすることで、世帯高所得を算出している
        対象者である = 所得降順 == 0

        控除後所得 = 対象世帯.members("児童扶養手当の控除後所得", 対象期間)
        return 対象世帯.sum(対象者である * 控除後所得)


class 児童扶養手当の控除後所得(Variable):
    value_type = float
    entity = 人物
    definition_period = DAY
    label = "各種控除が適用された後の児童扶養手当の世帯高所得額"
    reference = "https://www.city.otsu.lg.jp/soshiki/015/1406/g/jidofuyoteate/1389538447829.html"

    def formula(対象人物, 対象期間, parameters):
        所得 = 対象人物("所得", 対象期間)
        社会保険料 = parameters(対象期間).所得.社会保険料相当額
        給与所得及び雑所得からの控除額 = parameters(対象期間).所得.給与所得及び雑所得からの控除額
        障害者控除 = 対象人物("障害者控除", 対象期間)
        勤労学生控除 = 対象人物("勤労学生控除", 対象期間)
        配偶者特別控除 = 対象人物("配偶者特別控除", 対象期間)

        # 他の控除（雑損控除・医療費控除等）は定額でなく実費を元に算出するため除外する
        # 養育者が児童の父母の場合は寡婦控除・ひとり親控除は加えられない
        総控除額 = 社会保険料 + 給与所得及び雑所得からの控除額 + 障害者控除 + 勤労学生控除 + 配偶者特別控除

        # 負の数にならないよう、0円未満になった場合は0円に補正
        return np.clip(所得 - 総控除額, 0.0, None)


class 特別児童扶養手当の控除後世帯高所得(Variable):
    value_type = float
    entity = 世帯
    definition_period = DAY
    label = "各種控除が適用された後の特別児童扶養手当における世帯高所得額"
    reference = "https://www.city.otsu.lg.jp/soshiki/015/1406/g/jidofuyoteate/1389538447829.html"

    def formula(対象世帯, 対象期間, parameters):
        所得 = 対象世帯.members("所得", 対象期間)
        所得降順 = 対象世帯.get_rank(対象世帯, -所得)
        # 最も所得が大きい世帯員を対象とすることで、世帯高所得を算出している
        対象者である = 所得降順 == 0

        控除後所得 = 対象世帯.members("特別児童扶養手当の控除後所得", 対象期間)

        return 対象世帯.sum(対象者である * 控除後所得)


class 特別児童扶養手当の控除後所得(Variable):
    value_type = float
    entity = 人物
    definition_period = DAY
    label = "各種控除が適用された後の特別児童扶養手当における所得額"
    reference = "https://www.city.otsu.lg.jp/soshiki/015/1406/g/jidofuyoteate/1389538447829.html"

    def formula(対象人物, 対象期間, parameters):
        所得 = 対象人物("所得", 対象期間)
        社会保険料 = parameters(対象期間).所得.社会保険料相当額
        給与所得及び雑所得からの控除額 = parameters(対象期間).所得.給与所得及び雑所得からの控除額
        障害者控除 = 対象人物("障害者控除", 対象期間)
        勤労学生控除 = 対象人物("勤労学生控除", 対象期間)
        ひとり親控除 = 対象人物("ひとり親控除", 対象期間)
        寡婦控除 = 対象人物("寡婦控除", 対象期間)
        配偶者特別控除 = 対象人物("配偶者特別控除", 対象期間)

        # 他の控除（雑損控除・医療費控除等）は定額でなく実費を元に算出するため除外する
        総控除額 = 社会保険料 + 給与所得及び雑所得からの控除額 + 障害者控除 + 勤労学生控除 + ひとり親控除 + 寡婦控除 + 配偶者特別控除

        # 負の数にならないよう、0円未満になった場合は0円に補正
        return np.clip(所得 - 総控除額, 0.0, None)



"""
特定疾病療養の実装
"""

from openfisca_core.periods import DAY
from openfisca_core.variables import Variable
from openfisca_japan.entities import 世帯, 人物


class 特定疾病療養の対象者がいる(Variable):
    value_type = bool
    entity = 世帯
    definition_period = DAY
    label = "特定疾病療養の対象者がいるか否か"
    reference = "https://www.kyoukaikenpo.or.jp/shibu/shizuoka/cat080/20130225001/"

    def formula(対象世帯, 対象期間, _parameters):
        特定疾病療養の対象者である = 対象世帯.members("特定疾病療養の対象者である", 対象期間)
        return 対象世帯.sum(特定疾病療養の対象者である)


class 特定疾病療養の対象者である(Variable):
    value_type = int
    entity = 人物
    definition_period = DAY
    label = "特定疾病療養の対象者であるか否か"
    reference = "https://www.kyoukaikenpo.or.jp/shibu/shizuoka/cat080/20130225001/"
    documentation = """
    自己負担限度額を定める制度であり他制度の計算結果と合計できないため、対象者であるかどうかのみ計算

    条件は以下リンクも参考になる。
    https://hyogo-kenchiku-kenpo.or.jp/wp-content/Document/特定疾病療養留意事項.pdf
    """

    def formula(対象人物, 対象期間, _parameters):
        血友病の可能性がある = 対象人物("血友病の可能性がある", 対象期間)
        血友病関連条件 = 血友病の可能性がある

        HIV感染者である = 対象人物("HIV感染者である", 対象期間)
        血液製剤の投与によってHIVに感染した = 対象人物("血液製剤の投与によってHIVに感染した", 対象期間)
        HIV関連条件 = HIV感染者である * 血液製剤の投与によってHIVに感染した

        慢性腎不全である = 対象人物("慢性腎不全である", 対象期間)
        人工透析を行っている = 対象人物("人工透析を行っている", 対象期間)
        腎不全関連条件 = 慢性腎不全である * 人工透析を行っている

        return 血友病関連条件 + HIV関連条件 + 腎不全関連条件



"""
身体障害者手帳の実装
"""
from openfisca_core.indexed_enums import Enum
from openfisca_core.periods import DAY
from openfisca_core.variables import Variable
from openfisca_japan.entities import 人物


class 身体障害者手帳等級パターン(Enum):
    __order__ = "無 一級 二級 三級 四級 五級 六級 七級"
    無 = "無"
    一級 = "一級"
    二級 = "二級"
    三級 = "三級"
    四級 = "四級"
    五級 = "五級"
    六級 = "六級"
    七級 = "七級"


class 身体障害者手帳等級(Variable):
    value_type = Enum
    possible_values = 身体障害者手帳等級パターン
    default_value = 身体障害者手帳等級パターン.無
    entity = 人物
    definition_period = DAY
    label = "人物の身体障害者手帳等級"

    # NOTE: 交付年月日・有効期間は考慮しない



"""
ひとり親の実装
"""

from openfisca_core.periods import DAY
from openfisca_core.variables import Variable
from openfisca_japan.entities import 世帯, 人物


class 配偶者がいるがひとり親に該当(Variable):
    value_type = bool
    default_value = False
    entity = 世帯
    definition_period = DAY
    label = "配偶者がいるがひとり親に該当"


class ひとり親(Variable):
    value_type = bool
    entity = 世帯
    definition_period = DAY
    label = "ひとり親に該当するか否か"
    reference = "https://www.city.shibuya.tokyo.jp/kodomo/teate/hitorioya/hitorioya_teate.html"
    documentation = """
    渋谷区の児童扶養手当制度

    - 〒150-8010 東京都渋谷区宇田川町1-1
    - 渋谷区子ども青少年課子育て給付係
    - 03-3463-2558
    """

    def formula(対象世帯, 対象期間, parameters):
        配偶者がいない = 対象世帯.nb_persons(世帯.親) == 1

        return 配偶者がいない + 対象世帯("配偶者がいるがひとり親に該当", 対象期間)


class 夫と離別死別(Variable):
    value_type = bool
    default_value = False
    entity = 世帯
    definition_period = DAY
    label = "夫と離別・死別しているか否か"
    reference = "https://www.nta.go.jp/taxes/shiraberu/taxanswer/shotoku/1170.htm"


class 寡婦(Variable):
    value_type = bool
    entity = 人物
    definition_period = DAY
    label = "寡婦に該当するか否か"
    reference = "https://www.nta.go.jp/taxes/shiraberu/taxanswer/shotoku/1170.htm"

    def formula(対象人物, 対象期間, _parameters):
        子供がいない = 対象人物.世帯.sum(対象人物.has_role(世帯.子)) == 0
        親である = 対象人物.has_role(世帯.親)
        return 子供がいない * 親である * 対象人物.世帯("夫と離別死別", 対象期間)



"""
災害援護資金の実装
"""

from functools import cache

import numpy as np
from openfisca_core.periods import DAY
from openfisca_core.variables import Variable
from openfisca_japan import COUNTRY_DIR
from openfisca_japan.entities import 世帯
from openfisca_japan.variables.災害.住宅 import 住宅被害パターン, 家財の損害パターン, 災害による負傷の療養期間パターン


@cache
def 災害援護資金貸付限度額():
    """
    csvファイルから値を読み込み

    災害援護資金貸付限度額()[災害による負傷の療養期間, 住宅への損害] の形で参照可能
    """
    return np.genfromtxt(COUNTRY_DIR + "/assets/災害/支援/災害援護資金貸付限度額.csv",
                  delimiter=",", skip_header=1, dtype="int64")[:, 1:]


class 災害援護資金(Variable):
    value_type = int
    entity = 世帯
    definition_period = DAY
    label = "災害援護資金"
    reference = "https://www.bousai.go.jp/taisaku/hisaisyagyousei/pdf/kakusyuseido_tsuujou.pdf"
    documentation = """
    算出方法は以下リンクも参考になる。
    https://www.mhlw.go.jp/shinsai_jouhou/saigaishien.html
    """

    def formula(対象世帯, 対象期間, _parameters):
        災害救助法の適用地域である = 対象世帯("災害救助法の適用地域である", 対象期間)

        家財の損害 = 対象世帯("家財の損害", 対象期間)
        住宅被害 = 対象世帯("住宅被害", 対象期間)

        所得一覧 = 対象世帯.members("所得", 対象期間)
        所得降順 = 対象世帯.get_rank(対象世帯, -所得一覧)
        # NOTE: 厳密には世帯主は所得の多寡のみでは決まらないが、便宜上所得が最も多い世帯員を世帯主とする
        世帯主である = 所得降順 == 0

        災害による負傷の療養期間 = 対象世帯.members("災害による負傷の療養期間", 対象期間)

        世帯主が療養一か月以上の負傷をしている = 対象世帯.any((災害による負傷の療養期間 == 災害による負傷の療養期間パターン.一か月以上) * 世帯主である)

        災害による負傷の療養期間区分 = np.select(
            [世帯主が療養一か月以上の負傷をしている],
            [1],
            0).astype(int)

        住宅への損害区分 = np.select(
            [(家財の損害 == 家財の損害パターン.三分の一以上) * (住宅被害 == 住宅被害パターン.無),
             (住宅被害 == 住宅被害パターン.中規模半壊) + (住宅被害 == 住宅被害パターン.大規模半壊),
             (住宅被害 == 住宅被害パターン.全壊),
             # 住宅の解体が必要な場合厳密には半壊だった場合のみ金額が異なるが、解体前の住居の状態を入力上識別できないため一律同額とする
             # https://www.mhlw.go.jp/shinsai_jouhou/dl/shikingaiyou.pdf
             # 災害援護資金の条件に長期避難は記載されていないが、住宅被害パターンから1つのみを選択するため加える
             (住宅被害 == 住宅被害パターン.滅失または流失) + (住宅被害 == 住宅被害パターン.解体) + (住宅被害 == 住宅被害パターン.長期避難)],
            [1, 2, 3, 4],
            0).astype(int)

        災害援護資金_所得制限額 = 対象世帯("災害援護資金_所得制限額", 対象期間)
        所得制限以下である = 対象世帯.sum(所得一覧) <= 災害援護資金_所得制限額

        災害援護資金額 = 災害援護資金貸付限度額()[災害による負傷の療養期間区分, 住宅への損害区分]

        return 災害救助法の適用地域である * 所得制限以下である * 災害援護資金額


class 災害援護資金_所得制限額(Variable):
    value_type = int
    entity = 世帯
    definition_period = DAY
    label = "災害援護資金における所得制限額"
    reference = "https://www.bousai.go.jp/taisaku/hisaisyagyousei/pdf/kakusyuseido_tsuujou.pdf"
    documentation = """
    「住居が滅失した場合」には全壊、全焼、流出も含まれる
    https://www.mhlw.go.jp/bunya/seikatsuhogo/dl/saigaikyujo6h_03.pdf
    """

    def formula(対象世帯, 対象期間, parameters):
        世帯人数 = 対象世帯("世帯人数", 対象期間)
        住宅被害 = 対象世帯("住宅被害", 対象期間)

        世帯人数ごとの所得制限額 = np.array(parameters(対象期間).災害.支援.災害援護資金.所得制限額)  # 複数世帯入力(2以上の長さのndarray入力)対応のためndarray化
        所得制限額_一人当たり追加額 = parameters(対象期間).災害.支援.災害援護資金.所得制限額_一人当たり追加額
        所得制限額_住居が滅失した場合 = parameters(対象期間).災害.支援.災害援護資金.所得制限額_住居が滅失した場合

        所得制限額 = np.select(
            [(住宅被害 == 住宅被害パターン.滅失または流失) + (住宅被害 == 住宅被害パターン.全壊),
             世帯人数 <= 4,
             世帯人数 > 4],
            [所得制限額_住居が滅失した場合,
             世帯人数ごとの所得制限額[np.clip(世帯人数, 0, 4)],  # HACK: out of rangeを防ぐためインデックスを4以下に制限（np.selectは条件に合わない式も計算されるため）
             世帯人数ごとの所得制限額[4] + (世帯人数 - 4) * 所得制限額_一人当たり追加額],
            0).astype(int)

        return 所得制限額



"""
This file defines variables for the modelled legislation.

A variable is a property of an Entity such as a 人物, a 世帯…

See https://openfisca.org/doc/key-concepts/variables.html
"""

from functools import cache
import json

import numpy as np
# Import from openfisca-core the Python objects used to code the legislation in OpenFisca
from openfisca_core.periods import DAY
from openfisca_core.variables import Variable
# Import the Entities specifically defined for this tax and benefit system
from openfisca_japan import COUNTRY_DIR
from openfisca_japan.entities import 世帯, 人物


@cache
def 市区町村級地区分_キー一覧():
    """
    jsonファイルからキーを取得

    各要素は [都道府県名, 市区町村名] の形式
    """
    with open(COUNTRY_DIR + "/assets/市区町村級地区分.json") as f:
        d = json.load(f)
        キー一覧 = []
        for 都道府県名, 都道府県データ in d.items():
            for 市区町村名 in 都道府県データ.keys():
                キー一覧.append([都道府県名, 市区町村名])
        return np.array(キー一覧)


@cache
def 市区町村級地区分_値一覧():
    """
    jsonファイルから値を取得

    市区町村級地区分_値一覧[市区町村級地区分キー, 区分]の形式で取得可能
    """
    with open(COUNTRY_DIR + "/assets/市区町村級地区分.json") as f:
        d = json.load(f)
        値一覧 = []
        for 都道府県データ in d.values():
            for 市区町村データ in 都道府県データ.values():
                値一覧.append(市区町村データ)
        return np.array(値一覧)


class 居住都道府県(Variable):
    value_type = str
    entity = 世帯
    label = "居住都道府県"
    definition_period = DAY
    default_value = "北海道"


class 居住市区町村(Variable):
    value_type = str
    entity = 世帯
    label = "居住市区町村"
    definition_period = DAY
    default_value = "その他"


class 居住級地区分1(Variable):
    # m級地-n のとき m を返す
    value_type = int
    entity = 世帯
    label = "居住級地区分1"
    definition_period = DAY
    reference = "https://best-selection.co.jp/media/wp-content/uploads/2021/03/seikatsuhogo-kyuchi2022.pdf"

    def formula(対象世帯, 対象期間, _parameters):
        居住都道府県 = 対象世帯("居住都道府県", 対象期間)
        居住市区町村 = 対象世帯("居住市区町村", 対象期間)

        級地区分キー一覧 = 市区町村級地区分_キー一覧()
        級地区分インデックス = np.select(
            [(居住都道府県 == キー[0]) * (居住市区町村 == キー[1]) for キー in 級地区分キー一覧],
            list(range(len(級地区分キー一覧))),
            -1).astype(int)

        # NOTE: 市区町村級地区分()[級地区分インデックス, 0] が級地区分1を表す
        区分 = 市区町村級地区分_値一覧()[級地区分インデックス, 0]

        # 当てはまらない場合は3
        return np.select([級地区分インデックス != -1],
                         [区分],
                         3)


class 居住級地区分1_世帯員(Variable):
    # 世帯員ごとの所属世帯の居住級地区分 (障害者加算variableで世帯員ごとの身体障害者手帳等級との対応づけに必要)
    value_type = int
    entity = 人物
    label = "居住級地区分1_世帯員"
    definition_period = DAY
    reference = "https://best-selection.co.jp/media/wp-content/uploads/2021/03/seikatsuhogo-kyuchi2022.pdf"

    def formula(対象人物, 対象期間, _parameters):
        return 対象人物.世帯("居住級地区分1", 対象期間)


class 居住級地区分2(Variable):
    # m級地-n のとき n を返す
    value_type = int
    entity = 世帯
    label = "居住級地区分2"
    definition_period = DAY
    reference = "https://best-selection.co.jp/media/wp-content/uploads/2021/03/seikatsuhogo-kyuchi2022.pdf"

    def formula(対象世帯, 対象期間, parameters):
        居住都道府県 = 対象世帯("居住都道府県", 対象期間)
        居住市区町村 = 対象世帯("居住市区町村", 対象期間)

        級地区分キー一覧 = 市区町村級地区分_キー一覧()
        級地区分インデックス = np.select(
            [(居住都道府県 == キー[0]) * (居住市区町村 == キー[1]) for キー in 級地区分キー一覧],
            list(range(len(級地区分キー一覧))),
            -1).astype(int)

        # NOTE: 市区町村級地区分()[級地区分インデックス, 1] が級地区分2を表す
        区分 = 市区町村級地区分_値一覧()[級地区分インデックス, 1]

        # 当てはまらない場合は2
        return np.select(
            [級地区分インデックス != -1],
            [区分],
            2)


class 居住級地区分2_世帯員(Variable):
    # 世帯員ごとの所属世帯の居住級地区分
    value_type = int
    entity = 人物
    label = "居住級地区分2_世帯員"
    definition_period = DAY
    reference = "https://best-selection.co.jp/media/wp-content/uploads/2021/03/seikatsuhogo-kyuchi2022.pdf"

    def formula(対象人物, 対象期間, _parameters):
        return 対象人物.世帯("居住級地区分2", 対象期間)



"""
全制度で汎用的に使用するVariableを定義
"""

from openfisca_core.indexed_enums import Enum
from openfisca_core.periods import DAY
from openfisca_core.variables import Variable
from openfisca_japan.entities import 人物


class 小学生学年(Enum):
    __order__ = "一年生 二年生 三年生 四年生 五年生 六年生"
    一年生 = 1
    二年生 = 2
    三年生 = 3
    四年生 = 4
    五年生 = 5
    六年生 = 6


class 中学生学年(Enum):
    __order__ = "一年生 二年生 三年生"
    一年生 = 7
    二年生 = 8
    三年生 = 9


class 高校生学年(Enum):
    __order__ = "一年生 二年生 三年生"
    一年生 = 10
    二年生 = 11
    三年生 = 12


class 性別パターン(Enum):
    __order__ = "女性 男性 その他"
    女性 = "女性"
    男性 = "男性"
    その他 = "その他"


class 性別(Variable):
    value_type = Enum
    possible_values = 性別パターン
    default_value = 性別パターン.その他
    entity = 人物
    definition_period = DAY
    label = "人物の性別"


class 就労形態パターン(Enum):
    __order__ = "無 会社員 公務員 自営業 その他"
    無 = "無"
    会社員 = "会社員"
    公務員 = "公務員"
    自営業 = "自営業"
    その他 = "その他"


class 就労形態(Variable):
    value_type = Enum
    possible_values = 就労形態パターン
    default_value = 就労形態パターン.無
    entity = 人物
    definition_period = DAY
    label = "人物の就労形態"



"""
愛の手帳の実装

（知的障害者を対象とする手帳。東京都など以外では「療育手帳」の名称だが等級が異なる）
"""

from openfisca_core.indexed_enums import Enum
from openfisca_core.periods import DAY
from openfisca_core.variables import Variable
from openfisca_japan.entities import 人物


class 愛の手帳等級パターン(Enum):
    __order__ = "無 一度 二度 三度 四度"
    無 = "無"
    一度 = "一度"
    二度 = "二度"
    三度 = "三度"
    四度 = "四度"


class 愛の手帳等級(Variable):
    value_type = Enum
    possible_values = 愛の手帳等級パターン
    default_value = 愛の手帳等級パターン.無
    entity = 人物
    definition_period = DAY
    label = "人物の愛の手帳の等級"



"""
災害関連で共通して使用するVariableを定義
"""

from openfisca_core.periods import DAY
from openfisca_core.variables import Variable
from openfisca_japan.entities import 世帯


class 被災している(Variable):
    value_type = bool
    entity = 世帯
    definition_period = DAY
    label = "被災しているかどうか"



"""This file contains your country package's metadata and dependencies."""

from pathlib import Path

from setuptools import find_packages, setup

# Read the contents of our README file for PyPi
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text(encoding="utf-8")  # pylint: disable=W1514

setup(
    name = "OpenFisca-Japan",
    version = "1.3.0",
    author = "proj-inclusive",
    author_email = "proj.inclusive@gmail.com",
    classifiers = [
        "Development Status :: 5 - Production/Stable",
        "License :: OSI Approved :: GNU Affero General Public License v3",
        "Operating System :: POSIX",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Topic :: Scientific/Engineering :: Information Analysis",
        ],
    description = "OpenFisca tax and benefit system for Japan",
    long_description=long_description,
    long_description_content_type="text/markdown",
    keywords = "benefit microsimulation social tax",
    license = "http://www.fsf.org/licensing/licenses/agpl-3.0.html",
    license_files = ("LICENSE",),
    url = "https://github.com/project-inclusive/OpenFisca-Japan",
    include_package_data = True,  # Will read MANIFEST.in
    data_files = [
        (
            "share/openfisca/openfisca-country-template",
            ["README.md"],
            ),
        ],
    install_requires = [
        "OpenFisca-Core[web-api] >= 41.0.0, < 42.0.0",
        ],
    extras_require = {
        "dev": [
            "autopep8 >= 1.5.4, < 2.0.0",
            "flake8 >= 3.8.0, < 4.0.0",
            "flake8-bugbear >= 20.1.0, < 22.0.0",
            "flake8-builtins >= 1.5.0, < 2.0.0",
            "flake8-coding >= 1.3.0, < 2.0.0",
            "flake8-commas >= 2.0.0, < 3.0.0",
            "flake8-comprehensions >= 3.2.0, < 4.0.0",
            "flake8-docstrings >= 1.5.0, < 2.0.0",
            "flake8-import-order >= 0.18.0, < 1.0.0",
            "flake8-print >= 3.1.0, < 5.0.0",
            "flake8-quotes >= 3.2.0, < 4.0.0",
            "flake8-simplify >= 0.9.0, < 1.0.0",
            "flake8-use-fstring >= 1.1.0, < 2.0.0",
            "pylint >= 2.6.0, < 3.0.0",
            "pycodestyle >= 2.6.0, < 3.0.0",
            "python-dateutil > 2.8.1",
            "PyYAML == 6.0",
            "ruff == 0.1.9",
            ],
        },
    packages = find_packages(),
    )



"""
受験生チャレンジ支援貸付の実装
"""

import numpy as np
from openfisca_core.periods import DAY
from openfisca_core.variables import Variable
from openfisca_japan.entities import 世帯
from openfisca_japan.variables.全般 import 中学生学年, 高校生学年


class 受験生チャレンジ支援貸付(Variable):
    value_type = int
    default_value = 0
    entity = 世帯
    definition_period = DAY
    label = "受験生チャレンジ支援貸付"
    reference = "https://www.fukushi.metro.tokyo.lg.jp/seikatsu/teisyotokusyataisaku/jukenseichallenge.html"

    def formula(対象世帯, 対象期間, parameters):
        子供である = 対象世帯.has_role(世帯.子)
        学年 = 対象世帯.members("学年", 対象期間)

        中学三年生である = 学年 == 中学生学年.三年生.value
        高校三年生である = 学年 == 高校生学年.三年生.value

        学習塾等受講料 = 対象世帯.sum(子供である * (中学三年生である + 高校三年生である)) * parameters(対象期間).東京都.福祉.受験生チャレンジ支援貸付.学習塾等受講料
        高校受験料 = 対象世帯.sum(子供である * 中学三年生である) * parameters(対象期間).東京都.福祉.受験生チャレンジ支援貸付.高校受験料
        大学受験料 = 対象世帯.sum(子供である * 高校三年生である) * parameters(対象期間).東京都.福祉.受験生チャレンジ支援貸付.大学受験料

        年間支給金額 = 学習塾等受講料 + 高校受験料 + 大学受験料

        受験生チャレンジ支援貸付可能 = 対象世帯("受験生チャレンジ支援貸付可能", 対象期間)
        return 年間支給金額 * 受験生チャレンジ支援貸付可能


class 受験生チャレンジ支援貸付可能(Variable):
    value_type = int
    default_value = 0
    entity = 世帯
    definition_period = DAY
    label = "受験生チャレンジ支援貸付可能"
    reference = "https://www.fukushi.metro.tokyo.lg.jp/seikatsu/teisyotokusyataisaku/jukenseichallenge.html"

    def formula(対象世帯, 対象期間, _parameters):
        居住都道府県 = 対象世帯("居住都道府県", 対象期間)
        居住地条件 = 居住都道府県 == "東京都"

        預貯金一覧 = 対象世帯.members("預貯金", 対象期間)
        親または子である = 対象世帯.has_role(世帯.親) + 対象世帯.has_role(世帯.子)
        親子の預貯金総額 = 対象世帯.sum(預貯金一覧 * 親または子である)
        預貯金条件 = 親子の預貯金総額 <= 6000000

        ひとり親である = 対象世帯("ひとり親", 対象期間)
        世帯所得 = 対象世帯("世帯所得", 対象期間)
        世帯人数 = 対象世帯("世帯人数", 対象期間)

        ひとり親の場合の所得条件 =\
            ((世帯人数 == 2) * (世帯所得 <= 2805000)) +\
            ((世帯人数 == 3) * (世帯所得 <= 3532000)) +\
            ((世帯人数 == 4) * (世帯所得 <= 4175000)) +\
            ((世帯人数 == 5) * (世帯所得 <= 4674000))
        ひとり親でない場合の所得条件 =\
            ((世帯人数 == 3) * (世帯所得 <= 3087000)) +\
            ((世帯人数 == 4) * (世帯所得 <= 3599000)) +\
            ((世帯人数 == 5) * (世帯所得 <= 4149000)) +\
            ((世帯人数 == 6) * (世帯所得 <= 4776000))
        所得条件 = (ひとり親である * ひとり親の場合の所得条件) + (np.logical_not(ひとり親である) * ひとり親でない場合の所得条件)

        return 居住地条件 * 預貯金条件 * 所得条件



"""This file provides a function to load json example situations."""

import json
import os


DIR_PATH = os.path.dirname(os.path.abspath(__file__))


def parse(file_name):
    """Load json example situations."""
    file_path = os.path.join(DIR_PATH, file_name)
    with open(file_path, "r", encoding="utf8") as file:
        return json.loads(file.read())


独身 = parse("独身.json")
夫婦 = parse("夫婦.json")



"""
This file defines variables for the modelled legislation.

A variable is a property of an Entity such as a 人物, a 世帯…

See https://openfisca.org/doc/key-concepts/variables.html
"""

import numpy as np
# Import from openfisca-core the Python objects used to code the legislation in OpenFisca
from openfisca_core.holders import set_input_divide_by_period
from openfisca_core.periods import DAY
from openfisca_core.variables import Variable
# Import the Entities specifically defined for this tax and benefit system
from openfisca_japan.entities import 世帯, 人物


class 所得(Variable):
    # NOTE: 手当によって障害者控除や寡婦控除等の額を差し引く必要があるが、世帯情報が必要なため未実装
    value_type = float
    entity = 人物
    # NOTE: 所得自体は1年ごとに定義されるが、特定の日付における各種手当に計算できるように DAY で定義
    definition_period = DAY
    # Optional attribute. Allows user to declare a 所得 for a year.
    # OpenFisca will spread the yearly 金額 over the days contained in the year.
    set_input = set_input_divide_by_period
    label = "人物の所得"

    def formula(対象人物, 対象期間, _parameters):
        # NOTE: 収入660万円未満の場合給与所得控除額ではなく「所得税法別表第五」から算出するため、実際の所得と最大1000円程度誤差が発生する
        # https://www.nta.go.jp/taxes/shiraberu/taxanswer/shotoku/1410.htm
        収入 = 対象人物("収入", 対象期間)
        給与所得控除額 = 対象人物("給与所得控除額", 対象期間)
        # 負の数にならないよう、0円未満になった場合は0円に補正
        return np.clip(収入 - 給与所得控除額, 0.0, None)


class 収入(Variable):
    value_type = float
    entity = 人物
    # 年間収入を指す
    # NOTE: 収入自体は1年ごとに定義されるが、特定の日付における各種手当に計算できるように DAY で定義
    definition_period = DAY
    # Optional attribute. Allows user to declare this variable for a year.
    # OpenFisca will spread the yearly 金額 over the days contained in the year.
    set_input = set_input_divide_by_period
    label = "人物の収入"


class 世帯所得(Variable):
    value_type = float
    entity = 世帯
    definition_period = DAY
    label = "世帯全員の収入の合計"

    def formula(対象世帯, 対象期間, _parameters):
        各収入 = 対象世帯.members("所得", 対象期間)
        return 対象世帯.sum(各収入)


class 世帯高所得(Variable):
    value_type = float
    entity = 世帯
    definition_period = DAY
    label = "世帯で最も所得が高い人物の所得"

    def formula(対象世帯, 対象期間, _parameters):
        所得一覧 = 対象世帯.members("所得", 対象期間)
        return 対象世帯.max(所得一覧)



"""
特別児童扶養手当の実装
"""

import numpy as np
from openfisca_core.periods import DAY
from openfisca_core.variables import Variable
from openfisca_japan.entities import 世帯
from openfisca_japan.variables.障害.内部障害 import 内部障害パターン
from openfisca_japan.variables.障害.愛の手帳 import 愛の手帳等級パターン
from openfisca_japan.variables.障害.療育手帳 import 療育手帳等級パターン
from openfisca_japan.variables.障害.精神障害者保健福祉手帳 import 精神障害者保健福祉手帳等級パターン
from openfisca_japan.variables.障害.身体障害者手帳 import 身体障害者手帳等級パターン


class 特別児童扶養手当_最大(Variable):
    value_type = int
    entity = 世帯
    definition_period = DAY
    label = "保護者への特別児童扶養手当の最大額"
    reference = "https://www.mhlw.go.jp/bunya/shougaihoken/jidou/huyou.html"
    documentation = """
    特別児童扶養手当制度
    """

    def formula(対象世帯, 対象期間, parameters):
        特別児童扶養手当 = parameters(対象期間).福祉.育児.特別児童扶養手当

        身体障害者手帳等級一覧 = 対象世帯.members("身体障害者手帳等級", 対象期間)
        療育手帳等級一覧 = 対象世帯.members("療育手帳等級", 対象期間)
        愛の手帳等級一覧 = 対象世帯.members("愛の手帳等級", 対象期間)
        精神障害者保健福祉手帳等級一覧 = 対象世帯.members("精神障害者保健福祉手帳等級", 対象期間)
        内部障害一覧 = 対象世帯.members("内部障害", 対象期間)
        年齢 = 対象世帯.members("年齢", 対象期間)
        児童である = 対象世帯.has_role(世帯.子)
        上限年齢未満の児童 = 児童である * (年齢 < 特別児童扶養手当.上限年齢)

        # 精神障害者保健福祉手帳の等級は以下の中標津町のHPを参照
        # https://www.nakashibetsu.jp/kurashi/kosodate_fukushi/shougaisha/teate/tokubetujidou/
        # 内部障害は対象になる場合とならない場合があるため最大額の対象には含める
        対象障害者手帳等級 = \
            (身体障害者手帳等級一覧 == 身体障害者手帳等級パターン.一級) + \
            (身体障害者手帳等級一覧 == 身体障害者手帳等級パターン.二級) + \
            (身体障害者手帳等級一覧 == 身体障害者手帳等級パターン.三級) + \
            (療育手帳等級一覧 == 療育手帳等級パターン.A) + \
            (療育手帳等級一覧 == 療育手帳等級パターン.B) + \
            (愛の手帳等級一覧 == 愛の手帳等級パターン.一度) + \
            (愛の手帳等級一覧 == 愛の手帳等級パターン.二度) + \
            (愛の手帳等級一覧 == 愛の手帳等級パターン.三度) + \
            (精神障害者保健福祉手帳等級一覧 == 精神障害者保健福祉手帳等級パターン.一級) + \
            (精神障害者保健福祉手帳等級一覧 == 精神障害者保健福祉手帳等級パターン.二級) + \
            (内部障害一覧 == 内部障害パターン.有)

        対象児童人数 = 対象世帯.sum(上限年齢未満の児童 & 対象障害者手帳等級)

        手当条件 = 対象世帯("特別児童扶養手当の所得条件", 対象期間)
        手当金額 = 対象児童人数 * 特別児童扶養手当.金額.一級

        # TODO 障がいを事由とする公的年金を受けているときは対象にならない

        return 手当条件 * 手当金額


class 特別児童扶養手当_最小(Variable):
    value_type = int
    entity = 世帯
    definition_period = DAY
    label = "保護者への特別児童扶養手当の最小額"
    reference = "https://www.mhlw.go.jp/bunya/shougaihoken/jidou/huyou.html"
    documentation = """
    特別児童扶養手当制度
    """

    def formula(対象世帯, 対象期間, parameters):
        特別児童扶養手当 = parameters(対象期間).福祉.育児.特別児童扶養手当

        身体障害者手帳等級一覧 = 対象世帯.members("身体障害者手帳等級", 対象期間)
        療育手帳等級一覧 = 対象世帯.members("療育手帳等級", 対象期間)
        愛の手帳等級一覧 = 対象世帯.members("愛の手帳等級", 対象期間)
        精神障害者保健福祉手帳等級一覧 = 対象世帯.members("精神障害者保健福祉手帳等級", 対象期間)
        年齢 = 対象世帯.members("年齢", 対象期間)
        児童である = 対象世帯.has_role(世帯.子)
        上限年齢未満の児童 = 児童である * (年齢 < 特別児童扶養手当.上限年齢)

        # 精神障害者保健福祉手帳の等級は以下の中標津町のHPを参照
        # https://www.nakashibetsu.jp/kurashi/kosodate_fukushi/shougaisha/teate/tokubetujidou/
        # 内部障害は対象になる場合とならない場合があるため最小額の対象には含めない
        対象障害者手帳等級 = \
            (身体障害者手帳等級一覧 == 身体障害者手帳等級パターン.一級) + \
            (身体障害者手帳等級一覧 == 身体障害者手帳等級パターン.二級) + \
            (身体障害者手帳等級一覧 == 身体障害者手帳等級パターン.三級) + \
            (療育手帳等級一覧 == 療育手帳等級パターン.A) + \
            (療育手帳等級一覧 == 療育手帳等級パターン.B) + \
            (愛の手帳等級一覧 == 愛の手帳等級パターン.一度) + \
            (愛の手帳等級一覧 == 愛の手帳等級パターン.二度) + \
            (愛の手帳等級一覧 == 愛の手帳等級パターン.三度) + \
            (精神障害者保健福祉手帳等級一覧 == 精神障害者保健福祉手帳等級パターン.一級) + \
            (精神障害者保健福祉手帳等級一覧 == 精神障害者保健福祉手帳等級パターン.二級)

        対象児童人数 = 対象世帯.sum(上限年齢未満の児童 * 対象障害者手帳等級)

        手当条件 = 対象世帯("特別児童扶養手当の所得条件", 対象期間)
        手当金額 = 対象児童人数 * 特別児童扶養手当.金額.二級

        # TODO 障がいを事由とする公的年金を受けているときは対象にならない

        return 手当条件 * 手当金額


class 特別児童扶養手当の所得条件(Variable):
    value_type = bool
    entity = 世帯
    definition_period = DAY
    label = "保護者への特別児童扶養手当の所得条件"
    reference = "https://www.mhlw.go.jp/bunya/shougaihoken/jidou/huyou.html"
    documentation = """
    特別児童扶養手当制度
    """

    def formula(対象世帯, 対象期間, parameters):
        特別児童扶養手当 = parameters(対象期間).福祉.育児.特別児童扶養手当

        # 世帯で最も高い所得の人が基準となる
        世帯高所得 = 対象世帯("特別児童扶養手当の控除後世帯高所得", 対象期間)
        扶養人数 = 対象世帯("扶養人数", 対象期間)

        # 所得制限限度額の「扶養義務者」は、父または母か養育者が扶養義務者でない場合
        # 参考：厚労省HP (https://www.mhlw.go.jp/bunya/shougaihoken/jidou/huyou.html)
        # NOTE: 直接 `受給者[扶養人数]` のように要素参照すると型が合わず複数世帯の場合に計算できないためnp.selectを使用
        所得制限限度額 = np.select(
            [扶養人数 == i for i in range(10)],
            [特別児童扶養手当.所得制限限度額.受給者[i] for i in range(10)],
            -1).astype(int)

        # TODO: 所得税控除を世帯員ごとに計算できるようになったら以下のように修正
        # 児童扶養手当 = 対象世帯("児童扶養手当", 対象期間)
        # 所得制限限度額 = np.select(
        #    [児童扶養手当 <= 0, 0 < 児童扶養手当],
        #    [
        #        特別児童扶養手当.所得制限限度額.児童扶養手当受給者[扶養人数],
        #        特別児童扶養手当.所得制限限度額.扶養義務者[扶養人数],
        #    ],
        #    )
        所得条件 = 世帯高所得 < 所得制限限度額

        return 所得条件



"""
生活福祉資金貸付制度の実装
"""

import numpy as np
from openfisca_core.periods import DAY
from openfisca_core.variables import Variable
from openfisca_japan.entities import 世帯
from openfisca_japan.variables.障害.愛の手帳 import 愛の手帳等級パターン
from openfisca_japan.variables.障害.療育手帳 import 療育手帳等級パターン
from openfisca_japan.variables.障害.精神障害者保健福祉手帳 import 精神障害者保健福祉手帳等級パターン
from openfisca_japan.variables.障害.身体障害者手帳 import 身体障害者手帳等級パターン


class 障害者手帳を持つ世帯員がいる(Variable):
    value_type = bool
    entity = 世帯
    definition_period = DAY
    label = "障害者手帳を持つ世帯員がいる"

    def formula(対象世帯, 対象期間, parameters):
        身体障害者手帳等級一覧 = 対象世帯.members("身体障害者手帳等級", 対象期間)
        精神障害者保健福祉手帳等級一覧 = 対象世帯.members("精神障害者保健福祉手帳等級", 対象期間)
        療育手帳等級一覧 = 対象世帯.members("療育手帳等級", 対象期間)
        愛の手帳等級一覧 = 対象世帯.members("愛の手帳等級", 対象期間)

        障害者手帳を持つ世帯員 = ((身体障害者手帳等級一覧 != 身体障害者手帳等級パターン.無)
                       + (精神障害者保健福祉手帳等級一覧 != 精神障害者保健福祉手帳等級パターン.無)
                       + (療育手帳等級一覧 != 療育手帳等級パターン.無)
                       + (愛の手帳等級一覧 != 愛の手帳等級パターン.無))

        return 対象世帯.any(障害者手帳を持つ世帯員)


class 六十五歳以上の世帯員がいる(Variable):
    value_type = bool
    entity = 世帯
    definition_period = DAY
    label = "六十五歳以上の世帯員がいる"

    def formula(対象世帯, 対象期間, parameters):
        年齢 = 対象世帯.members("年齢", 対象期間)
        return 対象世帯.any(年齢 >= 65)


class 高校1年生以上の子供がいる(Variable):
    value_type = bool
    entity = 世帯
    definition_period = DAY
    label = "高校1年生以上の子供がいる"

    def formula(対象世帯, 対象期間, parameters):
        高校1年生以上である = 対象世帯.members("学年", 対象期間) >= 10
        子供である = 対象世帯.has_role(世帯.子)
        return 対象世帯.any(高校1年生以上である * 子供である)


class 中学3年生以上の子供がいる(Variable):
    value_type = bool
    entity = 世帯
    definition_period = DAY
    label = "中学3年生以上の子供がいる"

    def formula(対象世帯, 対象期間, parameters):
        中学3年生以上である = 対象世帯.members("学年", 対象期間) >= 9
        子供である = 対象世帯.has_role(世帯.子)
        return 対象世帯.any(中学3年生以上である * 子供である)


class 生活支援費(Variable):
    value_type = float
    entity = 世帯
    definition_period = DAY
    label = "生活支援費"
    reference = "https://www.mhlw.go.jp/stf/seisakunitsuite/bunya/hukushi_kaigo/seikatsuhogo/seikatsu-fukushi-shikin1/index.html"
    documentation = """
    詳細な金額、条件についてはこちらを参照。

    https://www.mhlw.go.jp/stf/seisakunitsuite/bunya/hukushi_kaigo/seikatsuhogo/seikatsu-fukushi-shikin1/kashitsukejoken.html
    """

    def formula(対象世帯, 対象期間, parameters):
        住民税非課税世帯 = 対象世帯("住民税非課税世帯", 対象期間)  # openfisca_japan/variables/所得.py の「住民税非課税世帯」を参照している
        障害者手帳を持つ世帯員がいる = 対象世帯("障害者手帳を持つ世帯員がいる", 対象期間)
        六十五歳以上の世帯員がいる = 対象世帯("六十五歳以上の世帯員がいる", 対象期間)

        貸付条件 = 住民税非課税世帯 + 障害者手帳を持つ世帯員がいる + 六十五歳以上の世帯員がいる

        # openfisca_japan/parameters/福祉/生活福祉資金貸付制度/生活支援費_貸付上限額_単身.yaml を参照している
        生活支援費_貸付上限額_単身 = parameters(対象期間).福祉.生活福祉資金貸付制度.生活支援費_貸付上限額_単身
        生活支援費_貸付上限額_二人以上 = parameters(対象期間).福祉.生活福祉資金貸付制度.生活支援費_貸付上限額_二人以上

        世帯人数 = 対象世帯("世帯人数", 対象期間)

        生活支援費_貸付上限額 = np.select(
            [世帯人数 == 1, 世帯人数 > 1],
            [生活支援費_貸付上限額_単身, 生活支援費_貸付上限額_二人以上],
            0)

        return 貸付条件 * 生活支援費_貸付上限額


class 一時生活再建費(Variable):
    value_type = float
    entity = 世帯
    definition_period = DAY
    label = "一時生活再建費"
    reference = "https://www.mhlw.go.jp/stf/seisakunitsuite/bunya/hukushi_kaigo/seikatsuhogo/seikatsu-fukushi-shikin1/index.html"
    documentation = """
    詳細な金額、条件についてはこちらを参照。

    https://www.mhlw.go.jp/stf/seisakunitsuite/bunya/hukushi_kaigo/seikatsuhogo/seikatsu-fukushi-shikin1/kashitsukejoken.html
    """

    def formula(対象世帯, 対象期間, parameters):
        住民税非課税世帯 = 対象世帯("住民税非課税世帯", 対象期間)  # openfisca_japan/variables/所得.py の「住民税非課税世帯」を参照している
        障害者手帳を持つ世帯員がいる = 対象世帯("障害者手帳を持つ世帯員がいる", 対象期間)
        六十五歳以上の世帯員がいる = 対象世帯("六十五歳以上の世帯員がいる", 対象期間)

        # openfisca_japan/parameters/福祉/生活福祉資金貸付制度/一時生活再建費_貸付上限額.yaml を参照している
        貸付条件 = 住民税非課税世帯 + 障害者手帳を持つ世帯員がいる + 六十五歳以上の世帯員がいる
        一時生活再建費_貸付上限額 = parameters(対象期間).福祉.生活福祉資金貸付制度.一時生活再建費_貸付上限額

        return 貸付条件 * 一時生活再建費_貸付上限額


class 福祉費(Variable):
    value_type = float
    entity = 世帯
    definition_period = DAY
    label = "福祉費"
    reference = "https://www.mhlw.go.jp/stf/seisakunitsuite/bunya/hukushi_kaigo/seikatsuhogo/seikatsu-fukushi-shikin1/index.html"

    def formula(対象世帯, 対象期間, _parameters):
        被災している = 対象世帯("被災している", 対象期間)
        福祉費_災害 = 対象世帯("福祉費_災害", 対象期間)
        福祉費_災害以外 = 対象世帯("福祉費_災害以外", 対象期間)

        # 被災している場合は災害関連の条件を優先して計算
        return np.where(被災している, 福祉費_災害, 福祉費_災害以外)


class 福祉費_災害(Variable):
    value_type = float
    entity = 世帯
    definition_period = DAY
    label = "福祉費（災害を受けたことにより臨時に必要となる経費）"
    reference = "https://www.mhlw.go.jp/stf/seisakunitsuite/bunya/hukushi_kaigo/seikatsuhogo/seikatsu-fukushi-shikin1/index.html"
    documentation = """
    福祉費の用途ごとの条件、金額はこちらを参照。
    https://www.mhlw.go.jp/bunya/seikatsuhogo/fukushihi.html
    """

    def formula(対象世帯, 対象期間, parameters):
        住民税非課税世帯 = 対象世帯("住民税非課税世帯", 対象期間)  # openfisca_japan/variables/所得.py の「住民税非課税世帯」を参照している
        障害者手帳を持つ世帯員がいる = 対象世帯("障害者手帳を持つ世帯員がいる", 対象期間)
        六十五歳以上の世帯員がいる = 対象世帯("六十五歳以上の世帯員がいる", 対象期間)

        # TODO: 災害によって特例措置が適用される場合その条件にも対応
        貸付条件 = 住民税非課税世帯 + 障害者手帳を持つ世帯員がいる + 六十五歳以上の世帯員がいる

        貸付上限額 = parameters(対象期間).福祉.生活福祉資金貸付制度.福祉費_災害_貸付上限額
        return 貸付条件 * 貸付上限額


# TODO: 用途ごとに条件と金額を細かく出し分ける
class 福祉費_災害以外(Variable):
    value_type = float
    entity = 世帯
    definition_period = DAY
    label = "福祉費（災害以外の場合）"
    reference = "https://www.mhlw.go.jp/stf/seisakunitsuite/bunya/hukushi_kaigo/seikatsuhogo/seikatsu-fukushi-shikin1/index.html"
    documentation = """
    福祉費の用途ごとの条件、金額はこちらを参照。
    https://www.mhlw.go.jp/bunya/seikatsuhogo/fukushihi.html
    """

    def formula(対象世帯, 対象期間, parameters):
        住民税非課税世帯 = 対象世帯("住民税非課税世帯", 対象期間)  # openfisca_japan/variables/所得.py の「住民税非課税世帯」を参照している
        障害者手帳を持つ世帯員がいる = 対象世帯("障害者手帳を持つ世帯員がいる", 対象期間)
        六十五歳以上の世帯員がいる = 対象世帯("六十五歳以上の世帯員がいる", 対象期間)

        # openfisca_japan/parameters/福祉/生活福祉資金貸付制度/福祉費_貸付上限額.yaml を参照している
        貸付条件 = 住民税非課税世帯 + 障害者手帳を持つ世帯員がいる + 六十五歳以上の世帯員がいる
        福祉費_貸付上限額 = parameters(対象期間).福祉.生活福祉資金貸付制度.福祉費_貸付上限額

        return 貸付条件 * 福祉費_貸付上限額


class 緊急小口資金(Variable):
    value_type = float
    entity = 世帯
    definition_period = DAY
    label = "緊急小口資金"
    reference = "https://www.mhlw.go.jp/stf/seisakunitsuite/bunya/hukushi_kaigo/seikatsuhogo/seikatsu-fukushi-shikin1/index.html"
    documentation = """
    詳細な金額、条件についてはこちらを参照。

    https://www.mhlw.go.jp/stf/seisakunitsuite/bunya/hukushi_kaigo/seikatsuhogo/seikatsu-fukushi-shikin1/kashitsukejoken.html
    """

    def formula(対象世帯, 対象期間, parameters):
        住民税非課税世帯 = 対象世帯("住民税非課税世帯", 対象期間)  # openfisca_japan/variables/所得.py の「住民税非課税世帯」を参照している
        障害者手帳を持つ世帯員がいる = 対象世帯("障害者手帳を持つ世帯員がいる", 対象期間)
        六十五歳以上の世帯員がいる = 対象世帯("六十五歳以上の世帯員がいる", 対象期間)

        # openfisca_japan/parameters/福祉/生活福祉資金貸付制度/緊急小口資金_貸付上限額.yaml を参照している
        貸付条件 = 住民税非課税世帯 + 障害者手帳を持つ世帯員がいる + 六十五歳以上の世帯員がいる
        緊急小口資金_貸付上限額 = parameters(対象期間).福祉.生活福祉資金貸付制度.緊急小口資金_貸付上限額

        return 貸付条件 * 緊急小口資金_貸付上限額


class 住宅入居費(Variable):
    value_type = float
    entity = 世帯
    definition_period = DAY
    label = "住宅入居費"
    reference = "https://www.mhlw.go.jp/stf/seisakunitsuite/bunya/hukushi_kaigo/seikatsuhogo/seikatsu-fukushi-shikin1/index.html"
    documentation = """
    詳細な金額、条件についてはこちらを参照。

    https://www.mhlw.go.jp/stf/seisakunitsuite/bunya/hukushi_kaigo/seikatsuhogo/seikatsu-fukushi-shikin1/kashitsukejoken.html
    """

    def formula(対象世帯, 対象期間, parameters):
        住民税非課税世帯 = 対象世帯("住民税非課税世帯", 対象期間)  # openfisca_japan/variables/所得.py の「住民税非課税世帯」を参照している
        障害者手帳を持つ世帯員がいる = 対象世帯("障害者手帳を持つ世帯員がいる", 対象期間)
        六十五歳以上の世帯員がいる = 対象世帯("六十五歳以上の世帯員がいる", 対象期間)

        # openfisca_japan/parameters/福祉/生活福祉資金貸付制度/住宅入居費_貸付上限額.yaml を参照している
        貸付条件 = 住民税非課税世帯 + 障害者手帳を持つ世帯員がいる + 六十五歳以上の世帯員がいる
        住宅入居費_貸付上限額 = parameters(対象期間).福祉.生活福祉資金貸付制度.住宅入居費_貸付上限額

        return 貸付条件 * 住宅入居費_貸付上限額


class 教育支援費(Variable):
    value_type = float
    entity = 世帯
    definition_period = DAY
    label = "教育支援費"
    reference = "https://www.mhlw.go.jp/stf/seisakunitsuite/bunya/hukushi_kaigo/seikatsuhogo/seikatsu-fukushi-shikin1/index.html"
    documentation = """
    詳細な金額、条件についてはこちらを参照。

    https://www.mhlw.go.jp/stf/seisakunitsuite/bunya/hukushi_kaigo/seikatsuhogo/seikatsu-fukushi-shikin1/kashitsukejoken.html
    """

    def formula(対象世帯, 対象期間, parameters):
        住民税非課税世帯 = 対象世帯("住民税非課税世帯", 対象期間)  # openfisca_japan/variables/所得.py の「住民税非課税世帯」を参照している
        高校1年生以上の子供がいる = 対象世帯("高校1年生以上の子供がいる", 対象期間)

        # openfisca_japan/parameters/福祉/生活福祉資金貸付制度/教育支援費_貸付上限額.yaml を参照している
        貸付条件 = 住民税非課税世帯 * 高校1年生以上の子供がいる
        教育支援費_貸付上限額 = parameters(対象期間).福祉.生活福祉資金貸付制度.教育支援費_貸付上限額

        return 貸付条件 * 教育支援費_貸付上限額


class 就学支度費(Variable):
    value_type = float
    entity = 世帯
    definition_period = DAY
    label = "就学支度費"
    reference = "https://www.mhlw.go.jp/stf/seisakunitsuite/bunya/hukushi_kaigo/seikatsuhogo/seikatsu-fukushi-shikin1/index.html"
    documentation = """
    詳細な金額、条件についてはこちらを参照。

    https://www.mhlw.go.jp/stf/seisakunitsuite/bunya/hukushi_kaigo/seikatsuhogo/seikatsu-fukushi-shikin1/kashitsukejoken.html
    """

    def formula(対象世帯, 対象期間, parameters):
        住民税非課税世帯 = 対象世帯("住民税非課税世帯", 対象期間)  # openfisca_japan/variables/所得.py の「住民税非課税世帯」を参照している
        中学3年生以上の子供がいる = 対象世帯("中学3年生以上の子供がいる", 対象期間)

        # openfisca_japan/parameters/福祉/生活福祉資金貸付制度/就学支度費_貸付上限額.yaml を参照している
        貸付条件 = 住民税非課税世帯 * 中学3年生以上の子供がいる
        就学支度費_貸付上限額 = parameters(対象期間).福祉.生活福祉資金貸付制度.就学支度費_貸付上限額

        return 貸付条件 * 就学支度費_貸付上限額


class 不動産担保型生活資金(Variable):
    value_type = float
    entity = 世帯
    definition_period = DAY
    label = "不動産担保型生活資金"
    reference = "https://www.mhlw.go.jp/stf/seisakunitsuite/bunya/hukushi_kaigo/seikatsuhogo/seikatsu-fukushi-shikin1/index.html"
    documentation = """
    詳細な金額、条件についてはこちらを参照。

    https://www.mhlw.go.jp/stf/seisakunitsuite/bunya/hukushi_kaigo/seikatsuhogo/seikatsu-fukushi-shikin1/kashitsukejoken.html
    """

    def formula(対象世帯, 対象期間, parameters):
        住民税非課税世帯 = 対象世帯("住民税非課税世帯", 対象期間)  # openfisca_japan/variables/所得.py の「住民税非課税世帯」を参照している
        六十五歳以上の世帯員がいる = 対象世帯("六十五歳以上の世帯員がいる", 対象期間)

        # openfisca_japan/parameters/福祉/生活福祉資金貸付制度/不動産担保型生活資金_貸付上限額.yaml を参照している
        貸付条件 = 住民税非課税世帯 * 六十五歳以上の世帯員がいる
        不動産担保型生活資金_貸付上限額 = parameters(対象期間).福祉.生活福祉資金貸付制度.不動産担保型生活資金_貸付上限額

        return 貸付条件 * 不動産担保型生活資金_貸付上限額
